/* Copyright (C) 2007 Timo Sirainen */

#include "lib.h"
#include "str.h"
#include "istream.h"
#include "index-mail.h"
#include "sql-istream.h"
#include "sql-storage.h"
#include "sql-storage-query.h"

#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>

struct mailbox_body_row {
	unsigned long long id;
	unsigned long long len;
};

static struct sql_field_def mailbox_body_row_defs[] = {
	SQL_DEF_STRUCT_ULLONG(id, mailbox_body_row),
	SQL_DEF_STRUCT_ULLONG(len, mailbox_body_row),

	{ 0, NULL, 0 }
};

#if 0
static time_t sql_mail_get_received_date(struct mail *_mail)
{
	struct index_mail *mail = (struct index_mail *)_mail;
	struct index_mail_data *data = &mail->data;
	struct stat st;
	uint32_t t;

	(void)index_mail_get_received_date(_mail);
	if (data->received_date != (time_t)-1)
		return data->received_date;

	if (sql_mail_stat(_mail, &st) < 0)
		return (time_t)-1;

	data->received_date = t = st.st_mtime;
	index_mail_cache_add(mail, MAIL_CACHE_RECEIVED_DATE, &t, sizeof(t));
	return data->received_date;
}

static time_t sql_mail_get_save_date(struct mail *_mail)
{
	struct index_mail *mail = (struct index_mail *)_mail;
	struct index_mail_data *data = &mail->data;
	struct stat st;
	uint32_t t;

	(void)index_mail_get_save_date(_mail);
	if (data->save_date != (time_t)-1)
		return data->save_date;

	if (sql_mail_stat(_mail, &st) < 0)
		return (time_t)-1;

	data->save_date = t = st.st_ctime;
	index_mail_cache_add(mail, MAIL_CACHE_SAVE_DATE, &t, sizeof(t));
	return data->save_date;
}

static uoff_t sql_mail_get_physical_size(struct mail *_mail)
{
	struct index_mail *mail = (struct index_mail *)_mail;
	struct index_mail_data *data = &mail->data;
	struct stat st;

	(void)index_mail_get_physical_size(_mail);
	if (data->physical_size != (uoff_t)-1)
		return data->physical_size;

	if (sql_mail_stat(_mail, &st) < 0)
		return (time_t)-1;

	index_mail_cache_add(mail, MAIL_CACHE_PHYSICAL_FULL_SIZE,
			     &data->physical_size, sizeof(data->physical_size));
	return data->physical_size;
}
#endif

static int
sql_mail_get_stream(struct mail *_mail, struct message_size *hdr_size,
		    struct message_size *body_size, struct istream **stream_r)
{
	struct index_mail *mail = (struct index_mail *)_mail;
	struct sql_mailbox *mbox = (struct sql_mailbox *)mail->ibox;
	const struct var_expand_table *variables;
	struct mailbox_body_row row;
	string_t *query_template;
	int ret;

	if (mail->data.stream == NULL) {
		variables = var_expand_table_build('m', dec2str(mbox->id),
						   'u', dec2str(_mail->uid),
						   '\0');

		ret = sql_storage_query_row(mbox->storage, SQ_MAILBOX_BODY_INIT,
					    variables, mailbox_body_row_defs,
					    &row, sizeof(row), NULL);
		if (ret <= 0) {
			if (ret == 0)
				mail_set_expunged(_mail);
			return -1;
		}

		variables = var_expand_table_build('m', dec2str(mbox->id),
						   'i', dec2str(row.id),
						   '\0');
		query_template = t_str_new(256);
		var_expand(query_template,
			   sql_storage_queries[SQ_MAILBOX_BODY_TEMPLATE].query,
			   variables);
		mail->data.stream =
			sql_istream_create(mbox->storage->db, FALSE, row.len,
					   str_c(query_template),
					   MAIL_READ_BLOCK_SIZE);
	}

	return index_mail_init_stream(mail, hdr_size, body_size, stream_r);
}

struct mail_vfuncs sql_mail_vfuncs = {
	index_mail_close,
	index_mail_free,
	index_mail_set_seq,
	index_mail_set_uid,

	index_mail_get_flags,
	index_mail_get_keywords,
	index_mail_get_parts,
	index_mail_get_date,
	index_mail_get_received_date,
	index_mail_get_save_date,
	index_mail_get_physical_size, /* physical = virtual in our case */
	index_mail_get_physical_size,
	index_mail_get_first_header,
	index_mail_get_headers,
	index_mail_get_header_stream,
	sql_mail_get_stream,
	index_mail_get_special,
	index_mail_update_flags,
	index_mail_update_keywords,
	index_mail_expunge
};
