#ifndef __SQL_STORAGE_H
#define __SQL_STORAGE_H

#include "index-storage.h"
#include "mailbox-list-private.h"

#define SQL_STORAGE_NAME "sql"
#define SQL_KEYWORD_COUNT 64

struct sql_storage {
	struct mail_storage storage;
	struct sql_db *db;
	unsigned int user_id;
};

struct sql_mailbox {
	struct index_mailbox ibox;
	struct sql_storage *storage;

	unsigned long long id;
	unsigned int uid_validity;
};

struct sql_transaction_context {
	struct index_transaction_context ictx;
	union mail_index_transaction_module_context module_ctx;

	struct sql_save_context *save_ctx;
};

extern struct mail_storage sql_storage;
extern struct mail_vfuncs sql_mail_vfuncs;

void sql_transaction_class_init(void);
void sql_transaction_class_deinit(void);

struct mailbox_list_context *
sql_mailbox_list_init(struct mail_storage *storage,
		      const char *ref, const char *mask,
		      enum mailbox_list_flags flags);
int sql_mailbox_list_deinit(struct mailbox_list_context *ctx);
struct mailbox_list *sql_mailbox_list_next(struct mailbox_list_context *ctx);

int sql_save_init(struct mailbox_transaction_context *_t,
		  enum mail_flags flags, struct mail_keywords *keywords,
		  time_t received_date, int timezone_offset,
		  const char *from_envelope, struct istream *input,
		  struct mail *dest_mail, struct mail_save_context **ctx_r);
int sql_save_continue(struct mail_save_context *ctx);
int sql_save_finish(struct mail_save_context *ctx);
void sql_save_cancel(struct mail_save_context *ctx);

int sql_transaction_save_commit_pre(struct sql_save_context *ctx);
void sql_transaction_save_commit_post(struct sql_save_context *ctx);
void sql_transaction_save_rollback(struct sql_save_context *ctx);

#endif
